#ifndef vtk_expat_mangle_h
#define vtk_expat_mangle_h

#define XML_DefaultCurrent vtkexpat_XML_DefaultCurrent
#define XML_ErrorString vtkexpat_XML_ErrorString
#define XML_ExpatVersion vtkexpat_XML_ExpatVersion
#define XML_ExpatVersionInfo vtkexpat_XML_ExpatVersionInfo
#define XML_ExternalEntityParserCreate vtkexpat_XML_ExternalEntityParserCreate
#define XML_FreeContentModel vtkexpat_XML_FreeContentModel
#define XML_GetBase vtkexpat_XML_GetBase
#define XML_GetBuffer vtkexpat_XML_GetBuffer
#define XML_GetCurrentByteCount vtkexpat_XML_GetCurrentByteCount
#define XML_GetCurrentByteIndex vtkexpat_XML_GetCurrentByteIndex
#define XML_GetCurrentColumnNumber vtkexpat_XML_GetCurrentColumnNumber
#define XML_GetCurrentLineNumber vtkexpat_XML_GetCurrentLineNumber
#define XML_GetErrorCode vtkexpat_XML_GetErrorCode
#define XML_GetFeatureList vtkexpat_XML_GetFeatureList
#define XML_GetIdAttributeIndex vtkexpat_XML_GetIdAttributeIndex
#define XML_GetInputContext vtkexpat_XML_GetInputContext
#define XML_GetParsingStatus vtkexpat_XML_GetParsingStatus
#define XML_GetSpecifiedAttributeCount vtkexpat_XML_GetSpecifiedAttributeCount
#define XML_MemFree vtkexpat_XML_MemFree
#define XML_MemMalloc vtkexpat_XML_MemMalloc
#define XML_MemRealloc vtkexpat_XML_MemRealloc
#define XML_Parse vtkexpat_XML_Parse
#define XML_ParseBuffer vtkexpat_XML_ParseBuffer
#define XML_ParserCreate vtkexpat_XML_ParserCreate
#define XML_ParserCreate_MM vtkexpat_XML_ParserCreate_MM
#define XML_ParserCreateNS vtkexpat_XML_ParserCreateNS
#define XML_ParserFree vtkexpat_XML_ParserFree
#define XML_ParserReset vtkexpat_XML_ParserReset
#define XML_ResumeParser vtkexpat_XML_ResumeParser
#define XML_SetAttlistDeclHandler vtkexpat_XML_SetAttlistDeclHandler
#define XML_SetBase vtkexpat_XML_SetBase
#define XML_SetBillionLaughsAttackProtectionActivationThreshold vtkexpat_XML_SetBillionLaughsAttackProtectionActivationThreshold
#define XML_SetBillionLaughsAttackProtectionMaximumAmplification vtkexpat_XML_SetBillionLaughsAttackProtectionMaximumAmplification
#define XML_SetCdataSectionHandler vtkexpat_XML_SetCdataSectionHandler
#define XML_SetCharacterDataHandler vtkexpat_XML_SetCharacterDataHandler
#define XML_SetCommentHandler vtkexpat_XML_SetCommentHandler
#define XML_SetDefaultHandler vtkexpat_XML_SetDefaultHandler
#define XML_SetDefaultHandlerExpand vtkexpat_XML_SetDefaultHandlerExpand
#define XML_SetDoctypeDeclHandler vtkexpat_XML_SetDoctypeDeclHandler
#define XML_SetElementDeclHandler vtkexpat_XML_SetElementDeclHandler
#define XML_SetElementHandler vtkexpat_XML_SetElementHandler
#define XML_SetEncoding vtkexpat_XML_SetEncoding
#define XML_SetEndCdataSectionHandler vtkexpat_XML_SetEndCdataSectionHandler
#define XML_SetEndDoctypeDeclHandler vtkexpat_XML_SetEndDoctypeDeclHandler
#define XML_SetEndElementHandler vtkexpat_XML_SetEndElementHandler
#define XML_SetEndNamespaceDeclHandler vtkexpat_XML_SetEndNamespaceDeclHandler
#define XML_SetEntityDeclHandler vtkexpat_XML_SetEntityDeclHandler
#define XML_SetExternalEntityRefHandler vtkexpat_XML_SetExternalEntityRefHandler
#define XML_SetExternalEntityRefHandlerArg vtkexpat_XML_SetExternalEntityRefHandlerArg
#define XML_SetHashSalt vtkexpat_XML_SetHashSalt
#define XML_SetNamespaceDeclHandler vtkexpat_XML_SetNamespaceDeclHandler
#define XML_SetNotationDeclHandler vtkexpat_XML_SetNotationDeclHandler
#define XML_SetNotStandaloneHandler vtkexpat_XML_SetNotStandaloneHandler
#define XML_SetParamEntityParsing vtkexpat_XML_SetParamEntityParsing
#define XML_SetProcessingInstructionHandler vtkexpat_XML_SetProcessingInstructionHandler
#define XML_SetReturnNSTriplet vtkexpat_XML_SetReturnNSTriplet
#define XML_SetSkippedEntityHandler vtkexpat_XML_SetSkippedEntityHandler
#define XML_SetStartCdataSectionHandler vtkexpat_XML_SetStartCdataSectionHandler
#define XML_SetStartDoctypeDeclHandler vtkexpat_XML_SetStartDoctypeDeclHandler
#define XML_SetStartElementHandler vtkexpat_XML_SetStartElementHandler
#define XML_SetStartNamespaceDeclHandler vtkexpat_XML_SetStartNamespaceDeclHandler
#define XML_SetUnknownEncodingHandler vtkexpat_XML_SetUnknownEncodingHandler
#define XML_SetUnparsedEntityDeclHandler vtkexpat_XML_SetUnparsedEntityDeclHandler
#define XML_SetUserData vtkexpat_XML_SetUserData
#define XML_SetXmlDeclHandler vtkexpat_XML_SetXmlDeclHandler
#define XML_StopParser vtkexpat_XML_StopParser
#define XML_UseForeignDTD vtkexpat_XML_UseForeignDTD
#define XML_UseParserAsHandlerArg vtkexpat_XML_UseParserAsHandlerArg

#endif
