// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
#ifndef vtkFeatures_h
#define vtkFeatures_h

/* Should all New methods use the object factory override. */
/* #undef VTK_ALL_NEW_OBJECT_FACTORY */

/* Whether VTK supports an extended memory space via memkind. */
/* #undef VTK_USE_MEMKIND */

#endif
