// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
#ifndef vtkIOMovieConfigure_h
#define vtkIOMovieConfigure_h

// If defined, `vtkAVIWriter.h` is available.
/* #undef VTK_USE_VIDEO_FOR_WINDOWS */

// If defined, `vtkMP4Writer.h` is available.
/* #undef VTK_USE_MICROSOFT_MEDIA_FOUNDATION */

#endif
