// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
#ifndef vtk_kwiml_h
#define vtk_kwiml_h

/* Use the KWIML library configured for VTK.  */
#include "vtkkwiml/abi.h"
#include "vtkkwiml/int.h"

#endif
